"""Main entrypoint into package."""
import warnings
from importlib import metadata
from typing import Any

try:
    __version__ = metadata.version(__package__)
except metadata.PackageNotFoundError:
    # Case where package metadata is not available.
    __version__ = "0.0.18"
del metadata  # optional, avoids polluting the results of dir(__package__)


def __getattr__(name: str) -> Any:
    if name == "RESTServer":
        warnings.warn(
            "Use of RESTServer is deprecated. Please update your code to use ServerClient instead.")

        from .server_resources.server_client import ServerClient
        return ServerClient
    elif name == "ServerClient":
        from .server_resources.server_client import ServerClient
        return ServerClient
    elif name == "ModelEngine":
        from .py_client.gaas.model import ModelEngine
        return ModelEngine
    elif name == "StorageEngine":
        from .py_client.gaas.storage import StorageEngine
        return StorageEngine
    elif name == "DatabaseEngine":
        from .py_client.gaas.database import DatabaseEngine
        return DatabaseEngine
    elif name == "VectorEngine":
        from .py_client.gaas.vector import VectorEngine
        return VectorEngine
    elif name == "FunctionEngine":
        from .py_client.gaas.function import FunctionEngine
        return FunctionEngine
    else:
        raise AttributeError(f"Could not find: {name}")


__all__ = [
    "RESTServer",
    "ServerClient",
    "ModelEngine",
    "StorageEngine",
    "DatabaseEngine",
    "VectorEngine",
]



import unittest
import pandas as pd
from ai_server.py_client.gaas.database import DatabaseEngine
from test_base_connection import TestServerClient
from variables import DATABASE_ENGINE_ID


class DatabaseTests(TestServerClient):

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.db_engine = DatabaseEngine(
            engine_id=DATABASE_ENGINE_ID,
        )

    def test_db_query(self):
        db_pandas = self.db_engine.execQuery('select * from diabetes')

        # Assert that the response is a dictionary with specific keys
        self.assertIsInstance(db_pandas, pd.DataFrame)


if __name__ == '__main__':
    unittest.main()



import unittest
import os
from test_base_connection import TestServerClient
from ai_server.py_client.gaas.vector import VectorEngine

from ai_server.tests.variables import VECTOR_ENGINE_ID


class VectorTests(TestServerClient):

    vector_engine = None
    test_files = None

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.vector_engine = VectorEngine(
            engine_id=VECTOR_ENGINE_ID,
        )

        if cls.test_files is None:
            CURRENT_DIR = os.path.dirname(os.path.abspath(__file__))
            TEST_FILE_DIR = os.path.join(CURRENT_DIR, "test_files")
            # List files with full paths
            TEST_FILES = [os.path.join(TEST_FILE_DIR, file)
                          for file in os.listdir(TEST_FILE_DIR)]
            cls.test_files = TEST_FILES

    def test_vector_search(self):
        from pathlib import Path

        self.vector_engine.addDocument(
            file_paths=self.test_files
        )

        file_names = [Path(file).name for file in self.test_files]

        vector_search = self.vector_engine.nearestNeighbor(
            search_statement='How did the WHO improve access to oxygen supplies?',
            limit=5
        )

        # Assert that the response is a dictionary with specific keys
        self.assertIsInstance(vector_search, list)

        top_match = vector_search[0]
        self.assertIsInstance(top_match, dict)

        self.assertCountEqual(top_match.keys(), [
                              'Score', 'Source', 'Modality', 'Divider', 'Part', 'Tokens', 'Content'])

        self.vector_engine.removeDocument(file_names=file_names)

    def test_list_documents(self):

        document_info_list = self.vector_engine.listDocuments()[0]

        self.assertIsInstance(document_info_list, list)

        if len(document_info_list) > 0:
            document_info = document_info_list[0]
            self.assertIsInstance(document_info, dict)
            self.assertIn('fileName', document_info)


if __name__ == '__main__':
    unittest.main()



import unittest
from ai_server.py_client.gaas.model import ModelEngine
from ai_server.py_client.gaas.vector import VectorEngine
from test_base_connection import TestServerClient
from variables import LLM_CHAT_ENGINE_ID, VECTOR_ENGINE_ID
from langchain.schema import HumanMessage
from langchain_core.outputs import (
    ChatGeneration,
    ChatResult,
)
import os
from langchain_core.documents import Document


class LangChainTests(TestServerClient):

    def test_langchain_chat_model(self):
        model = ModelEngine(
            engine_id=LLM_CHAT_ENGINE_ID,
        )

        langchain_model = model.to_langchain_chat_model()

        messages = [HumanMessage(content='what is the capital of france ey?')]

        model_response = langchain_model._generate(messages=messages)

        # Check that we are returning a langchain ChatResult class object
        self.assertIsInstance(model_response, ChatResult)

        # Check the ChatResult object has the expected attributes
        self.assertTrue(hasattr(model_response, 'generations'))
        self.assertTrue(hasattr(model_response, 'llm_output'))

        # Check that the generations attribute contains a message
        for generation in model_response.generations:
            self.assertIsInstance(generation, ChatGeneration)
            self.assertTrue(hasattr(generation, 'message'))
            self.assertIsNotNone(generation.message)

    def test_langchain_vector_model(self):
        model = VectorEngine(
            engine_id=VECTOR_ENGINE_ID,
        )

        langchain_model = model.to_langchain_vector_store()

        CURRENT_DIR = os.path.dirname(os.path.abspath(__file__))
        TEST_FILE_DIR = os.path.join(CURRENT_DIR, "test_files")
        TEST_FILES = [os.path.join(TEST_FILE_DIR, file)
                      for file in os.listdir(TEST_FILE_DIR)]

        langchain_model.add_documents(
            file_paths=TEST_FILES
        )

        sim_search = langchain_model.similarity_search(
            query="checks and balances", limit=5)

        self.assertIsInstance(sim_search, list)

        top_match = sim_search[0]
        self.assertIsInstance(top_match, Document)


if __name__ == '__main__':

    unittest.main()



from dotenv import load_dotenv
import os

# curr_file_path = os.path.dirname(os.path.abspath(__file__))
# dotenv_path = os.path.join(curr_file_path, '.env')
if load_dotenv():
    print(f"Loaded environment variables")
else:
    print(f"Failed to load environment variables")

# SWAP THESE WITH YOUR OWN VALUES IF TESTING LOCALLY
local_creds = {
    'SECRET_KEY': os.getenv('LOCAL_SECRET_KEY'),
    'ACCESS_KEY': os.getenv('LOCAL_ACCESS_KEY'),
    'ENDPOINT': "http://localhost:9090/Monolith_Dev/api",
    'LLM_CHAT_ENGINE_ID': '4801422a-5c62-421e-a00c-05c6a9e15de8',
    'LLM_EMBEDDING_ENGINE_ID': '',
    'VECTOR_ENGINE_ID': 'dae9096d-1891-4077-b8bc-63ecdce08e14',
    'DATABASE_ENGINE_ID': '515721c0-340f-42fa-b3ff-138882b8b75b'
}

# Don't change these values unless there are problems ie. permissions, etc.
dev_creds = {
    'SECRET_KEY': os.getenv('DEV_SECRET_KEY'),
    'ACCESS_KEY': os.getenv('DEV_ACCESS_KEY'),
    'ENDPOINT': "https://workshop.cfg.deloitte.com/cfg-ai-dev/Monolith/api",
    'LLM_CHAT_ENGINE_ID': '4801422a-5c62-421e-a00c-05c6a9e15de8',
    'LLM_EMBEDDING_ENGINE_ID': 'e4449559-bcff-4941-ae72-0e3f18e06660',
    'VECTOR_ENGINE_ID': '1222b449-1bc6-4358-9398-1ed828e4f26a',
    'DATABASE_ENGINE_ID': '950eb187-e352-444d-ad6a-6476ed9390af',
    'STORAGE_ENGINE_ID': '2d905aa3-b703-4c98-8133-5bcaefddac1e',
}

# SWAP THIS VALUE TO CHANGE BETWEEN LOCAL AND DEV CREDENTIALS
active_creds = dev_creds

# These are the variables that will be used in the tests
SECRET_KEY = active_creds['SECRET_KEY']
ACCESS_KEY = active_creds['ACCESS_KEY']
ENDPOINT = active_creds['ENDPOINT']
LLM_CHAT_ENGINE_ID = active_creds['LLM_CHAT_ENGINE_ID']
LLM_EMBEDDING_ENGINE_ID = active_creds['LLM_EMBEDDING_ENGINE_ID']
VECTOR_ENGINE_ID = active_creds['VECTOR_ENGINE_ID']
DATABASE_ENGINE_ID = active_creds['DATABASE_ENGINE_ID']
STORAGE_ENGINE_ID = active_creds['STORAGE_ENGINE_ID']



from setuptools import find_packages, setup
import os

# Read the contents of README.md if it exists
if os.path.exists('README.md'):
    with open('README.md', 'r') as f:
        long_description = f.read()
else:
    long_description = "AI Server SDK - Python client SDK to connect to the AI Server."

setup(
    name="ai-server-sdk",
    version="0.0.18",
    packages=find_packages(),
    install_requires=[
        'requests',
        'pandas',
        'jsonpickle',
    ],
    extras_require={
        "full": ["langchain", "langchain-community"]
    },
    author="Thomas Trankle, Maher Khalil, Ryan Weiler",
    description="Utility package to connect to AI Server instances.",
    long_description=long_description,
    long_description_content_type='text/markdown',
    license="MIT",
)



import unittest
from ai_server.py_client.gaas.storage import StorageEngine
from test_base_connection import TestServerClient
from variables import STORAGE_ENGINE_ID


class StorageTests(TestServerClient):

    def test_storage_list(self):
        storage = StorageEngine(
            engine_id=STORAGE_ENGINE_ID,
        )

        storage_list = storage.list(storagePath='/my-new-test-folder/')

        self.assertIsInstance(storage_list, list)

        if len(storage_list) > 0:
            self.assertIsInstance(storage_list[0], str)

    def test_storage_list_details(self):
        storage = StorageEngine(
            engine_id=STORAGE_ENGINE_ID,
        )

        storage_list = storage.listDetails(storagePath='/my-new-test-folder/')

        self.assertIsInstance(storage_list, list)

        if len(storage_list) > 0:
            self.assertIsInstance(storage_list[0], dict)

            expected_keys = {'Path', 'Name', 'Size',
                             'MimeType', 'ModTime', 'IsDir', 'Tier'}

            self.assertCountEqual(storage_list[0].keys(), expected_keys)


if __name__ == '__main__':
    unittest.main()



from typing import List, Dict, Optional
import json
import logging
from ai_server.server_resources.server_proxy import ServerProxy

logger: logging.Logger = logging.getLogger(__name__)


class VectorEngine(ServerProxy):
    """Python class to interact with Vector Database Engines defined in CFG AI"""

    engine_type = "VECTOR"

    def __init__(
        self,
        engine_id: str,
        insight_id: Optional[str] = None,
    ):
        super().__init__()
        self.engine_id = engine_id
        self.insight_id = insight_id

        logger.info("VectorEngine initialized with engine id " + engine_id)

    def addDocument(
        self,
        file_paths: List[str],
        param_dict: Optional[Dict] = {},
        insight_id: Optional[str] = None,
    ) -> bool:
        """
        This method is used to add documents to a vector database. The engine itself will determine how the the documents are
        processed and the embeddings are created.

        Args:
            file_paths (`List[str]`): List of local files paths to push to the server and index.
            param_dict (`Optional[Dict]`): Additional parameters the engine might need to process the documents.
            insight_id (`Optional[str]`): The insight ID to upload the documents to and process the request. Default is to use the clients current insight.
        """
        if insight_id is None:
            if self.insight_id is None:
                insight_id = self.insight_id
            else:
                insight_id = self.server.cur_insight

        assert self.server is not None
        insight_files = self.server.upload_files(
            files=file_paths,
            insight_id=insight_id,
        )

        pixel = (
            'CreateEmbeddingsFromDocuments=(engine="'
            + self.engine_id
            + '", filePaths='
            + json.dumps(insight_files)
        )

        if len(param_dict) != 0:
            pixel += ", paramValues = " + json.dumps(param_dict)

        pixel += ");"

        output_payload_message = self.server.run_pixel(
            payload=pixel, insight_id=insight_id, full_response=True
        )

        if output_payload_message["pixelReturn"][0]["operationType"] == ["ERROR"]:
            raise RuntimeError(output_payload_message["pixelReturn"][0]["output"])

        return output_payload_message["pixelReturn"][0]["output"]

    def addVectorCSVFile(
        self,
        file_paths: List[str],
        param_dict: Optional[Dict] = {},
        insight_id: Optional[str] = None,
    ) -> bool:
        """
        Add the vector csv file format documents into the vector database

        Args:
            file_paths (`List[str]`):  The paths (relative to the insight_id) of the files to add
            param_dict (`dict`): A dictionary with optional parameters for listing the documents (index class for FAISS as an example)
            insight_id (`Optional[str]`): Unique identifier for the temporal worksapce where actions are being isolated
        """
        assert file_paths is not None
        if insight_id is None:
            insight_id = self.insight_id

        optionalParams = (
            f",paramValues=[{param_dict}]" if param_dict is not None else ""
        )

        pixel = f'CreateEmbeddingsFromVectorCSVFile(engine="{self.engine_id}",filePaths={file_paths}{optionalParams});'

        output_payload_message = self.server.run_pixel(
            payload=pixel, insight_id=insight_id, full_response=True
        )

        if output_payload_message["pixelReturn"][0]["operationType"] == ["ERROR"]:
            raise RuntimeError(output_payload_message["pixelReturn"][0]["output"])

        return output_payload_message["pixelReturn"][0]["output"]

    def removeDocument(
        self,
        file_names: List[str],
        param_dict: Optional[Dict] = {},
        insight_id: Optional[str] = None,
    ) -> bool:
        """
        Remove the documents from the vector database

        Args:
            file_names (`List[str]`):  The names of the files to remove
            param_dict (`dict`): A dictionary with optional parameters for listing the documents (index class for FAISS as an example)
            insight_id (`Optional[str]`): Unique identifier for the temporal worksapce where actions are being isolated
        """
        assert file_names is not None
        if insight_id is None:
            insight_id = self.insight_id

        optionalParams = (
            f",paramValues=[{param_dict}]" if param_dict is not None else ""
        )

        pixel = f'RemoveDocumentFromVectorDatabase(engine="{self.engine_id}",fileNames={file_names}{optionalParams});'

        output_payload_message = self.server.run_pixel(
            payload=pixel, insight_id=insight_id, full_response=True
        )

        if output_payload_message["pixelReturn"][0]["operationType"] == ["ERROR"]:
            raise RuntimeError(output_payload_message["pixelReturn"][0]["output"])

        return output_payload_message["pixelReturn"][0]["output"]

    def nearestNeighbor(
        self,
        search_statement: str,
        limit: Optional[int] = 5,
        filters: Optional[Dict] | Optional[str] = None,
        filters_str: Optional[str] = None,
        metafilters: Optional[Dict] | Optional[str] = None,
        metafilters_str: Optional[str] = None,
        param_dict: Optional[Dict] = {},
        insight_id: Optional[str] = None,
    ) -> List[Dict]:
        """
        Perform a nearest neighbor or semantic search against a vector database. The searchStatement will be
        converted to a vector using the same embedding model utilized to create the document(s) embeddings.

        Args:
            search_statement (`str`): The statement to search for semantic matches in the vector database
            limit (`Optional[int]`): The amount of top matches to return
            filters (`Optional[Dict]`): A dictionary of filters to apply to the search results
            filters_str (`Optional[str]`): A string of filters to apply to the search results
            metafilters (`Optional[Dict]`): A dictionary of metafilters to apply to the search results
            metafilters_str (`Optional[str]`): A string of metafilters to apply to the search results
            param_dict (`Optional[Dict]`): Additional parameters the engine might need to remove the documents.
            insight_id (`Optional[str]`): The insight ID to upload the documents to and process the request. Default is to use the clients current insight.

        Returns:
            `List[Dict]`: A list of dictionaries that contain the top semantic matches against the search statement
        """

        if insight_id is None:
            if self.insight_id is None:
                insight_id = self.insight_id
            else:
                insight_id = self.server.cur_insight

        pixel = f'VectorDatabaseQuery(engine = "{self.engine_id}", command = ["<e>{search_statement}</e>"], limit = {limit}'

        # 1. Check if filters_str parameter is provided (if so use this)
        # 2. If not, check if filters parameter is provided and check if it is a string (if so use this)
        # 3. If not, check if filters parameter is provided and check if it is a dictionary (if so build the string)
        optional_filters = ""
        if filters_str is not None:
            optional_filters = f",filters=[{filters_str}]"
        if filters is not None and optional_filters == "":
            if isinstance(filters, str):
                optional_filters = f",filters=[{filters}]"
            elif isinstance(filters, dict):
                filter_conditions = []
                for key, value in filters.items():
                    formatted_key = key.capitalize()
                    if isinstance(value, str):
                        formatted_values = f'"{value}"'
                    else:
                        formatted_values = ", ".join([f'"{v}"' for v in value])
                    filter_conditions.append(f"{formatted_key} == [{formatted_values}]")

                optional_filters = (
                    f",filters = [ Filter({', '.join(filter_conditions)})]"
                    if filter_conditions
                    else ""
                )

            else:
                raise ValueError(
                    "Invalid filters type. Filter must be string or dictionary"
                )

        # 1. Check if metafilters_str parameter is provided (if so use this)
        # 2. If not, check if metafilters parameter is provided and check if it is a string (if so use this)
        # 3. If not, check if metafilters parameter is provided and check if it is a dictionary (if so build the string)
        optional_meta_filters = ""
        if metafilters_str is not None:
            optional_meta_filters = f",metaFilters=[{metafilters_str}]"
        if metafilters is not None and optional_meta_filters == "":
            if isinstance(metafilters, str):
                optional_meta_filters = f",metaFilters=[{metafilters}]"
            elif isinstance(metafilters, dict):
                metafilter_conditions = []
                for key, value in metafilters.items():
                    formatted_key = key.capitalize()
                    if isinstance(value, str):
                        formatted_values = f'"{value}"'
                    else:
                        formatted_values = ", ".join([f'"{v}"' for v in value])
                    metafilter_conditions.append(
                        f"{formatted_key} == [{formatted_values}]"
                    )

                optional_meta_filters = (
                    f",metaFilters = [ Filter({', '.join(metafilter_conditions)})]"
                    if metafilter_conditions
                    else ""
                )

            else:
                raise ValueError(
                    "Invalid metafilters type. Metafilters must be string or dictionary"
                )

        pixel += optional_filters + optional_meta_filters

        if len(param_dict) != 0:
            pixel += ", paramValues = " + json.dumps(param_dict)

        pixel += ");"

        output_payload_message = self.server.run_pixel(
            payload=pixel, insight_id=insight_id, full_response=True
        )

        if output_payload_message["pixelReturn"][0]["operationType"] == ["ERROR"]:
            raise RuntimeError(output_payload_message["pixelReturn"][0]["output"])

        return output_payload_message["pixelReturn"][0]["output"]

    def listDocuments(
        self,
        param_dict: Optional[Dict] = {},
        insight_id: Optional[str] = None,
    ) -> List[Dict]:
        """
        List the documents in the vector database

        Args:
            param_dict (`dict`): A dictionary with optional parameters for listing the documents (index class for FAISS as an example)
            insight_id (`Optional[str]`): Unique identifier for the temporal worksapce where actions are being isolated
        """
        if insight_id is None:
            insight_id = self.insight_id

        optionalParams = (
            f",paramValues=[{param_dict}]" if param_dict is not None else ""
        )

        pixel = (
            f'ListDocumentsInVectorDatabase(engine="{self.engine_id}"{optionalParams});'
        )

        output_payload_message = self.server.run_pixel(
            payload=pixel, insight_id=insight_id, full_response=True
        )

        if output_payload_message["pixelReturn"][0]["operationType"] == ["ERROR"]:
            raise RuntimeError(output_payload_message["pixelReturn"][0]["output"])

        return output_payload_message["pixelReturn"][0]["output"]

    def to_langchain_vector_store(self):
        from langchain_core.callbacks import CallbackManagerForRetrieverRun
        from langchain_core.documents import Document
        from langchain_core.retrievers import BaseRetriever

        class VectorStoreCfgAI(BaseRetriever):
            engine_id: str
            vector_engine: VectorEngine
            insight_id: Optional[str]

            def __init__(self, vector_engine):
                super().__init__(
                    engine_id=vector_engine.engine_id,
                    insight_id=vector_engine.insight_id,
                    vector_engine=vector_engine,
                )

            def add_documents(self, file_paths: List[str]):
                """Adds documents to the vector store.

                Args:
                    file_paths (List[str]): Path to documents
                """
                self.vector_engine.addDocument(
                    file_paths=file_paths, insight_id=self.insight_id
                )

            def similarity_search(self, query: str, limit: int) -> List[Document]:
                """Performs a similarity search on the vector store using the given query.

                Args:
                    query (str): Query to search
                    limit (int): Maximum number of nearest neighbor results to be returned

                Returns:
                    List[Document]:  A list of the top `k` most similar documents to the query.
                    Each document contains the page content and metadata associated with
                    the corresponding search result.
                """
                results = self.vector_engine.nearestNeighbor(
                    search_statement=query, limit=limit, insight_id=self.insight_id
                )

                documents = [
                    Document(page_content=result["Content"], metadata=result)
                    for result in results
                ]
                return documents

            def _get_relevant_documents(
                self, query: str, *, run_manager: CallbackManagerForRetrieverRun
            ) -> List[Document]:
                return self.similarity_search(query)

            @property
            def _llm_type(self) -> str:
                """Return type of chat model."""
                return "CFG AI"

        return VectorStoreCfgAI(vector_engine=self)



import unittest
from typing import Generator
from ai_server.py_client.gaas.model import ModelEngine
from test_base_connection import TestServerClient
from variables import LLM_EMBEDDING_ENGINE_ID, LLM_CHAT_ENGINE_ID


class ModelTests(TestServerClient):

    def test_model_ask(self):
        model = ModelEngine(
            engine_id=LLM_CHAT_ENGINE_ID,
        )

        model_response_list = model.ask(
            question='what is the capital of france ey?')

        # Assert that the response is a dictionary with specific keys
        self.assertIsInstance(model_response_list, list)

        model_response = model_response_list[0]

        self.assertIsInstance(model_response, dict)

        self.assertCountEqual(model_response.keys(), [
                              'response', 'numberOfTokensInPrompt', 'numberOfTokensInResponse', 'messageId', 'roomId'])

    def test_model_ask_with_stream(self):

        model = ModelEngine(
            engine_id=LLM_CHAT_ENGINE_ID,
        )

        model_response_stream = model.stream_ask(
            question='what is the capital of france ey?')

        self.assertIsInstance(model_response_stream, Generator)

    def test_model_embeddings(self):
        model = ModelEngine(
            engine_id=LLM_EMBEDDING_ENGINE_ID,
        )

        model_response_list = model.embeddings(
            strings_to_embed=['what is the capital of france ey?'])

        # Assert that the response is a dictionary with specific keys
        self.assertIsInstance(model_response_list, list)

        model_response = model_response_list[0]
        self.assertIsInstance(model_response, dict)

        self.assertCountEqual(model_response.keys(), [
                              'response', 'numberOfTokensInPrompt', 'numberOfTokensInResponse'])


if __name__ == '__main__':
    # python test_model.py -v -m "access_key=my_access_key secret_key=my_secret_key base=http://example.com/api"

    unittest.main()



from typing import Optional
import logging
from ai_server.server_resources.server_proxy import ServerProxy

logger: logging.Logger = logging.getLogger(__name__)


class DatabaseEngine(ServerProxy):
    def __init__(self, engine_id: str, insight_id: Optional[str] = None):
        super().__init__()

        self.engine_id = engine_id
        self.insight_id = insight_id

        logger.info("DatabaseEngine initialized with engine id " + engine_id)

    def execQuery(
        self,
        query: str,
        insight_id: Optional[str] = None,
        return_pandas: Optional[bool] = True,
        server_output_format: Optional[str] = "json",
    ):
        """
        Connect to a database an execute SQL against it to create a pandas frame

        Args:
            query (`str`):
                A user's access key is a unique identifier used for authentication and authorization. It will allow users or applications to access specific resources or perform designated actions within an ai-server instance.
            insight_id (`str`):
                Unique identifier for the temporal worksapce where actions are being isolated
            return_pandas (`bool`):
                true/false flag for creating a pandas frame
            server_output_format (`str`):
                Define wheter to write the query result to a file or json. *Note*, if file is selected then its only accessible via server UI
        """
        if insight_id is None:
            insight_id = self.insight_id

        fileLoc = super().call(
            engine_type="database",
            engine_id=self.engine_id,
            insight_id=insight_id,
            method_name="execQuery",
            method_args=[query, server_output_format],
            method_arg_types=["java.lang.String", "java.lang.String"],
        )

        if isinstance(fileLoc, list) and len(fileLoc) > 0:
            fileLoc = fileLoc[0]

        if return_pandas:
            logger.info(f"The output is {fileLoc}")
            import pandas as pd

            logger.info(fileLoc)
            if isinstance(fileLoc, dict) and len(fileLoc) > 0:
                rows = []
                # saftey check based on how java gson structure the response
                if "myArrayList" in fileLoc.keys() and {
                    rows.append(d[key]) if key == "map" else "notMap"
                    for d in fileLoc["myArrayList"]
                    for key in d.keys()
                } == {None}:
                    return pd.DataFrame.from_dict(rows)
            elif isinstance(fileLoc, str):
                return pd.read_json(fileLoc)
            else:
                return fileLoc
        else:
            if isinstance(fileLoc, dict) and len(fileLoc) > 0:
                return fileLoc
            else:
                return open(fileLoc, "r").read()

    def insertData(self, query: str, insight_id: Optional[str] = None, commit: bool = True) -> None:
        """
        Connect to a database an execute SQL against it to insert data

        Args:
            query (`str`): A SQL statement to insert values into a table
            insight_id (`Optional[str]`): Unique identifier for the temporal worksapce where actions are being isolated
            commit (`bool`): commit to the database if autocommit is false. default is true
        """
        return self.runQuery(query, insight_id, commit)

    def updateData(self, query: str, insight_id: Optional[str] = None, commit: bool = True) -> None:
        """
        Connect to a database an execute SQL against it to update data

        Args:
            query (`str`): A SQL statement to insert values into a table
            insight_id (`Optional[str]`): Unique identifier for the temporal worksapce where actions are being isolated
            commit (`bool`): commit to the database if autocommit is false. default is true
        """
        return self.runQuery(query, insight_id, commit)

    def removeData(self, query: str, insight_id: Optional[str] = None, commit: bool = True) -> None:
        """
        Connect to a database an execute SQL against it to delete/remove data

        Args:
            query (`str`): A SQL statement like DELETE FROM diab WHERE age=19
            insight_id (`Optional[str]`): Unique identifier for the temporal worksapce where actions are being isolated
            commit (`bool`): commit to the database if autocommit is false. default is true
        """
        return self.runQuery(query, insight_id, commit)

    def runQuery(self, query: str, insight_id: Optional[str] = None, commit: bool = True):
        """
        This will execute the ExecQuery() reactor for a database

        Args:
        query (`str`): A SQL statement like DELETE FROM diab WHERE age=19
        insight_id (`Optional[str]`): Unique identifier for the temporal worksapce where actions are being isolated
        """

        if insight_id is None:
            insight_id = self.insight_id

        commitStr = "true" if commit else "false"

        pixel = (
            f'Database(database = "{self.engine_id}")|Query("<encode>{query}</encode>")|ExecQuery(commit={commitStr});'
        )

        output_payload_message = self.server.run_pixel(payload=pixel, insight_id=insight_id, full_response=True)

        if output_payload_message["pixelReturn"][0]["operationType"] == ["ERROR"]:
            raise RuntimeError(output_payload_message["pixelReturn"][0]["output"])

        return output_payload_message["pixelReturn"][0]["output"]



from typing import Optional
import logging
from ai_server.server_resources.server_proxy import ServerProxy

logger: logging.Logger = logging.getLogger(__name__)


class FunctionEngine(ServerProxy):
    def __init__(self, engine_id: str, insight_id: Optional[str] = None):
        super().__init__()

        self.engine_id = engine_id
        self.insight_id = insight_id

        logger.info("FunctionEngine initialized with engine id " + engine_id)

    def execute(self, parameterMap: dict, insight_id: Optional[str] = None) -> None:
        """
        Connect to a function and execute

        Args:
            parameterMap (`dict`): A dictionary with the payload for the function engine
        """
        if insight_id is None:
            insight_id = self.insight_id

        pixel = f'ExecuteFunctionEngine(engine = "{self.engine_id}", map=[{parameterMap}]);'

        output_payload_message = self.server.run_pixel(payload=pixel, insight_id=insight_id, full_response=True)

        if output_payload_message["pixelReturn"][0]["operationType"] == ["ERROR"]:
            raise RuntimeError(output_payload_message["pixelReturn"][0]["output"])

        return output_payload_message["pixelReturn"][0]["output"]



from typing import Optional, Dict
import logging
from ai_server.server_resources.server_proxy import ServerProxy

logger: logging.Logger = logging.getLogger(__name__)


class StorageEngine(ServerProxy):
    def __init__(self, engine_id: str, insight_id: Optional[str] = None):
        super().__init__()

        self.engine_id = engine_id
        self.insight_id = insight_id

        logger.info(f"StorageEngine initialized with engine id {engine_id}")

    def __execute_pixel(self, pixel: str, insight_id: Optional[str] = None):
        if insight_id is None:
            insight_id = self.insight_id

        output_payload_message = self.server.run_pixel(payload=pixel, insight_id=insight_id, full_response=True)

        if output_payload_message["pixelReturn"][0]["operationType"] == ["ERROR"]:
            raise RuntimeError(output_payload_message["pixelReturn"][0]["output"])

        return output_payload_message["pixelReturn"][0]["output"]

    def list(self, storagePath: str, insight_id: Optional[str] = None):
        """
        This method is responsible for listing the files in the storage engine

        Args:
            storagePath (`str`): The path in the storage engine to list
            insight_id (`Optional[str]`): Unique identifier for the temporal worksapce where actions are being isolated

        Returns:
            list: list of files/folders in the given path of the storage engine
        """
        pixel = f'Storage("{self.engine_id}")|ListStoragePath(storagePath="{storagePath}");'
        return self.__execute_pixel(pixel, insight_id)

    def listDetails(self, storagePath: str, insight_id: Optional[str] = None):
        """
        This method is responsible for listing the files in the storage engine with extra details

        Args:
            storagePath (`str`): The path in the storage engine to list
            insight_id (`Optional[str]`): Unique identifier for the temporal worksapce where actions are being isolated

        Returns:
            list: list of files/folders in the given path of the storage engine with extra details
        """
        pixel = f'Storage("{self.engine_id}")|ListStoragePathDetails(storagePath="{storagePath}");'
        return self.__execute_pixel(pixel, insight_id)

    def syncLocalToStorage(
        self,
        storagePath: str,
        localPath: str,
        space: Optional[str] = None,
        metadata: Optional[Dict] = {},
        insight_id: Optional[str] = None,
    ):
        """
        This method is responsible for syncing from insight/project/user space to cloud storage

        Args:
            storagePath (`str`): The path in the storage engine to sync into
            localPath (`str`): The path in the application to sync from (insight, project, user space)
            space (`str`): The space to use. None = current insight. Can be the project id or 'user' for the user specific space
            metadata (`Optional[Dict]`): Define custom metadata associated with the files being moved to storage. Only available if the underlying storage system supports custom metadata.
            insight_id (`Optional[str]`): Unique identifier for the temporal worksapce where actions are being isolated

        Returns:
            boolean: true/false if this ran successfully
        """
        spaceStr = f',space="{space}"' if space is not None else ""
        metadataStr = f",metadata=[{metadata}]" if metadata is not None else ""
        pixel = f'Storage("{self.engine_id}")|SyncLocalToStorage(storagePath="{storagePath}",filePath="{localPath}"{spaceStr}{metadataStr});'

        return self.__execute_pixel(pixel, insight_id)

    def syncStorageToLocal(
        self, storagePath: str, localPath: str, space: Optional[str] = None, insight_id: Optional[str] = None
    ):
        """
        This method is responsible for syncing from cloud storage into the insight/project/user space

        Args:
            storagePath (`str`): The path in the storage engine to sync from
            localPath (`str`): The path in the application to sync into (insight, project, user space)
            space (`str`): The space to use. None = current insight. Can be the project id or 'user' for the user specific space
            insight_id (`Optional[str]`): Unique identifier for the temporal worksapce where actions are being isolated

        Returns:
            boolean: true/false if this ran successfully
        """
        spaceStr = f',space="{space}"' if space is not None else ""
        pixel = f'Storage("{self.engine_id}")|SyncStorageToLocal(storagePath="{storagePath}",filePath="{localPath}"{spaceStr});'

        return self.__execute_pixel(pixel, insight_id)

    def copyToLocal(
        self,
        storagePath: str,
        localPath: str,
        space: Optional[str] = None,
        insight_id: Optional[str] = None,
    ):
        """
        This method is responsible for copying from cloud storage into the insight/project/user space

        Args:
            storagePath (`str`): The path in the storage engine to pull from
            localPath (`str`): The path in the application to copy into (insight, project, user space)
            space (`str`): The space to use. None = current insight. Can be the project id or 'user' for the user specific space
            insight_id (`Optional[str]`): Unique identifier for the temporal worksapce where actions are being isolated

        Returns:
            boolean: true/false if this ran successfully
        """
        spaceStr = f',space="{space}"' if space is not None else ""
        pixel = f'Storage("{self.engine_id}")|PullFromStorage(storagePath="{storagePath}",filePath="{localPath}"{spaceStr});'

        return self.__execute_pixel(pixel, insight_id)

    def copyToStorage(
        self,
        storagePath: str,
        localPath: str,
        space: Optional[str] = None,
        metadata: Optional[Dict] = {},
        insight_id: Optional[str] = None,
    ):
        """
        This method is responsible for copying from insight/project/user space to cloud storage

        Args:
            storagePath (`str`): The path in the storage engine to push into
            localPath (`str`): The path in the application we are pushing to cloud storage (insight, project, user space)
            space (`str`): The space to use. None = current insight. Can be the project id or 'user' for the user specific space
            metadata (`Optional[Dict]`): Define custom metadata associated with the files being moved to storage. Only available if the underlying storage system supports custom metadata.
            insight_id (`Optional[str]`): Unique identifier for the temporal worksapce where actions are being isolated

        Returns:
            boolean: true/false if this ran successfully
        """
        spaceStr = f',space="{space}"' if space is not None else ""
        metadataStr = f",metadata=[{metadata}]" if metadata is not None else ""
        pixel = f'Storage("{self.engine_id}")|PushToStorage(storagePath="{storagePath}",filePath="{localPath}"{spaceStr}{metadataStr});'

        return self.__execute_pixel(pixel, insight_id)

    def deleteFromStorage(
        self, storagePath: str, leaveFolderStructure: Optional[bool] = False, insight_id: Optional[str] = None
    ):
        """
        This method is responsible for deleting from a storage

        Args:
            storagePath (`str`): The path in the storage engine to delete
            leaveFolderStructure (`bool`): If we should maintain the folder structure after deletion
            insight_id (`Optional[str]`): Unique identifier for the temporal worksapce where actions are being isolated

        Returns:
            boolean: true/false if this ran successfully
        """
        leaveFolderStructureStr = "true" if leaveFolderStructure else "false"
        pixel = f'Storage("{self.engine_id}")|DeleteFromStorage(storagePath="{storagePath}",leaveFolderStructure={leaveFolderStructureStr});'

        return self.__execute_pixel(pixel, insight_id)



from typing import List, Optional, Dict, Union, Any, Generator
import logging
from ai_server.server_resources.server_proxy import ServerProxy

logger: logging.Logger = logging.getLogger(__name__)


class ModelEngine(ServerProxy):
    def __init__(self, engine_id: Optional[str], insight_id: Optional[str] = None):
        super().__init__()
        self.engine_id = engine_id
        self.insight_id = insight_id

        logger.info("ModelEngine initialized with engine id " + engine_id)

    def ask(
        self,
        question: str,
        context: Optional[str] = None,
        insight_id: Optional[str] = None,
        param_dict: Optional[Dict] = None,
    ) -> List[Dict]:
        """
        Ask a question to a text-generation model hosted on the ai server.

        Args:
            question (`str`):
                The single string question you are asking a an LLM
            context (`str`):
                Set the given context string for an interaction with an LLM
            insight_id (`str`):
                Unique identifier for the temporal worksapce where actions are being isolated
            param_dict (`dict`):
                Additional inference params like temperature, max new tokens and top_k etc

                *NOTE* you can pass in
                param_dict = {"full_prompt":full_prompt, "temperature":temp, "max_new_tokens":max_token}
                where full_prompt is the an multi faceted prompt construct before sending the payload

                For OpenAI, this would be a list of dictionaris where the only keys within each dictionary are 'role' and 'content'
                For TextGen, this could be a list simialr to OpenAI or a complete string that has all the pieces pre constructed
        Response:
            `Union[Dict, Generator]`: A list that contains a dictioanry with the answer to the question, insight id and the message id
        """
        if insight_id is None:
            insight_id = self.insight_id

        optionalContext = f',context=["<encode>{context}</encode>"]' if (context is not None) else ""
        optionalParamDict = f",paramValues=[{param_dict}]" if (param_dict is not None) else ""

        pixel = f'LLM(engine="{self.engine_id}", command="<encode>{question}</encode>"{optionalContext}{optionalParamDict});'

        output_payload_message = self.server.run_pixel(payload=pixel, insight_id=insight_id, full_response=True)

        if output_payload_message["pixelReturn"][0]["operationType"] == ["ERROR"]:
            raise RuntimeError(output_payload_message["pixelReturn"][0]["output"])

        return output_payload_message["pixelReturn"][0]["output"]

    def stream_ask(
        self,
        question: str,
        context: Optional[str] = None,
        insight_id: Optional[str] = None,
        param_dict: Optional[Dict] = None,
    ) -> Generator:
        """
        Stream responses from a question asked to a text-generation model hosted on the ai server.

        Args:
            question (`str`):
                The single string question you are asking a an LLM
            context (`str`):
                Set the given context string for an interaction with an LLM
            insight_id (`str`):
                Unique identifier for the temporal worksapce where actions are being isolated
            param_dict (`dict`):
                Additional inference params like temperature, max new tokens and top_k etc

                *NOTE* you can pass in
                param_dict = {"full_prompt":full_prompt, "temperature":temp, "max_new_tokens":max_token}
                where full_prompt is the an multi faceted prompt construct before sending the payload

                For OpenAI, this would be a list of dictionaris where the only keys within each dictionary are 'role' and 'content'
                For TextGen, this could be a list simialr to OpenAI or a complete string that has all the pieces pre constructed
        Response:
            `Generator`: A generator to iterate through to get the partial responses
        """
        if insight_id is None:
            insight_id = self.insight_id

        optionalContext = f',context=["<encode>{context}</encode>"]' if (context is not None) else ""
        optionalParamDict = f",paramValues=[{param_dict}]" if (param_dict is not None) else ""

        pixel = f'LLM(engine="{self.engine_id}", command="<encode>{question}</encode>"{optionalContext}{optionalParamDict});'

        for message in self.server.get_partial_responses(
            self.server.run_pixel_separate_thread(payload=pixel, insight_id=insight_id, full_response=True)
        ):
            yield message

    def embeddings(
        self,
        strings_to_embed: List[str],
        insight_id: Optional[str] = None,
        param_dict: Optional[Dict] = None,
    ) -> Dict:
        """
        Pass in a list of strings to an embeddings model and get a list of vectors.

        Args:
            question (`str`):
                A user's access key is a unique identifier used for authentication and authorization. It will allow users or applications to access specific resources or perform designated actions within an ai-server instance.
            insight_id (`str`):
                Unique identifier for the temporal worksapce where actions are being isolated
            param_dict (`dict`):
                Optional

        Returns

        """
        if isinstance(strings_to_embed, str):
            strings_to_embed = [strings_to_embed]
        assert isinstance(strings_to_embed, list)

        if insight_id is None:
            if self.insight_id is None:
                insight_id = self.insight_id
            else:
                insight_id = self.server.cur_insight

        assert self.server is not None

        optionalParamDict = f",paramValues=[{param_dict}]" if (param_dict is not None) else ""

        pixel = f'Embeddings(engine="{self.engine_id}", values={strings_to_embed}{optionalParamDict});'

        output_payload_message = self.server.run_pixel(payload=pixel, insight_id=insight_id, full_response=True)

        if output_payload_message["pixelReturn"][0]["operationType"] == ["ERROR"]:
            raise RuntimeError(output_payload_message["pixelReturn"][0]["output"])

        return output_payload_message["pixelReturn"][0]["output"]

    def get_model_engine_id(self) -> str:
        return self.engine_id

    def to_langchain_embedder(self):
        """Transform the model engine into a langchain `Embeddings`object so that it can be used with langchain code"""

        from langchain_core.embeddings import Embeddings

        class CfgEmbeddingsEngine(Embeddings):
            def __init__(self, modelEngine):
                self.modelEngine = modelEngine

            def embed_documents(self, texts: List[str]) -> List[List[float]]:
                """Embed search docs."""
                return self.modelEngine.embeddings(strings_to_embed=texts)[0]["response"]

            def embed_query(self, text: str) -> List[float]:
                return self.modelEngine.embeddings(strings_to_embed=[text])[0]["response"][0]

        return CfgEmbeddingsEngine(modelEngine=self)

    def to_langchain_chat_model(self):
        """Transform the model engine into a langchain `BaseChatModel` object so that it can be used with langchain code"""
        from langchain_core.language_models.chat_models import BaseChatModel
        from langchain_core.outputs import (
            ChatGeneration,
            ChatResult,
        )
        from langchain_core.messages import (
            AIMessage,
            BaseMessage,
        )

        class ChatCfgAI(BaseChatModel):
            engine_id: str
            model_engine: ModelEngine
            model_type: str

            def __init__(self, model_engine):
                data = {
                    "engine_id": model_engine.get_model_engine_id(),
                    "model_engine": model_engine,
                    "model_type": model_engine.get_model_type()[0],
                }

                super().__init__(**data)

            class Config:
                """Configuration for this pydantic object."""

                allow_population_by_field_name = True

            def _generate(
                self,
                messages: List[BaseMessage],
                stop: Optional[List[str]] = None,
                **kwargs: Any,
            ) -> ChatResult:
                """Top Level call"""
                full_prompt = self.convert_messages_to_full_prompt(messages)
                response = self.model_engine.ask(question="", param_dict={**kwargs, **{"full_prompt": full_prompt}})

                return self._create_chat_result(response=response[0])

            def _create_chat_result(self, response: Dict[str, Any]) -> ChatResult:
                generations = []

                message = response.pop("response", "")
                generation_info = dict()
                if "logprobs" in response.keys():
                    generation_info["logprobs"] = response.pop("logprobs", {})
                gen = ChatGeneration(
                    message=AIMessage(content=message),
                    generation_info=generation_info,
                )

                generations.append(gen)

                return ChatResult(generations=generations, llm_output=response)

            def convert_messages_to_full_prompt(
                self,
                messages: List[BaseMessage],
            ) -> Union[Dict[str, Any], str]:
                """Convert a LangChain message to a the correct response for a model.

                Args:
                    message: The LangChain message.

                Returns:
                    The `Dict` or `str` containing the message payload.
                """

                if self.model_type in ["OPEN_AI", "VERTEX"]:
                    # assume this is a chat based openai model, otherwise why would you call this
                    # class
                    full_prompt: List[Dict[str, Any]]
                    from langchain_community.adapters.openai import (
                        convert_message_to_dict,
                    )

                    full_prompt = [convert_message_to_dict(m) for m in messages]
                    return full_prompt
                else:
                    full_prompt: str
                    full_prompt = "\n".join([m.content for m in messages])
                    return full_prompt

            @property
            def _llm_type(self) -> str:
                """Return type of chat model."""
                return "CFG AI"

        return ChatCfgAI(model_engine=self)



from typing import List, Dict, Any, Optional, Generator

import logging
import threading

logger: logging.Logger = logging.getLogger(__name__)


class ServerProxy:
    """This class is used to transform model, database, storage and vector payloads into PayloadStructs before sending it to the RemoteEngineRunReactor via the runPixel endpoint"""

    def __init__(
        self,
    ):
        """
        Initialize the ServerProxy instance.
        """
        self.epoc = 0
        self.condition = threading.Condition()
        from ai_server.server_resources.server_client import ServerClient

        self.server = ServerClient.da_server
        assert self.server is not None, "Please authenticate using your access and secret keys"
        logger.info("Server is set")

    def get_next_epoc(self) -> str:
        """This method atomically increments the epoc count by one plus the current value."""
        self.epoc = self.epoc + 1
        return f"py_{self.epoc}"

    def comm(
        self,
        epoc: str,
        engine_type: str,
        engine_id: str,
        method_name: str,
        method_args: Optional[List[Any]] = [],
        method_arg_types: Optional[List[str]] = [],
        insight_id: Optional[str] = None,
    ) -> None:
        """
        This method in responsible for:
            - converting the args into a PayloadStruct
            - adds itself to the clients monitor block
            - forwards the PayloadStruct to the client to be sent to the Tomcat Server

        Args:
            epoc (`str`): The epoc ID for the payload struct
            engine_type (`str`): The engine type that will be called from the tomcat server. Options are model, storage, database or vector and are set in NativePyEngineWorker.java
            engine_id (`str`): The unique identifier of the engine being called. This passed so the tomcat server can call Utility.java to find the engine
            method_name (`str`): The IEngine method name that is available in the engine_type
            method_args (`Optional[List[Any]]`): A list of object to be sent to the IEngine method as inputs
            method_arg_types (`Optional[List[str]]`): A list of Java class names that represent the method args types
            insight_id (`Optional[str]`): The unique identifier for the insight

        Returns:
            `None`
        """

        # converts this into a PayloadStruct
        payload = {
            "epoc": epoc,
            "response": False,
            "engineType": engine_type,
            "interim": False,
            "objId": engine_id,  # all the method stuff will come here and below
            "methodName": method_name,
            "payload": method_args,
            "payloadClassNames": method_arg_types,
            "insightId": insight_id if insight_id is not None else self.server.cur_insight,
            "operation": "ENGINE",
        }

        # adds itself to the monitor block
        self.server.monitors.update({epoc: self.condition})
        logger.info("The server is ServerClient object")
        self.server.send_request(payload)

    def call(
        self,
        engine_type: str,
        engine_id: str,
        method_name: str = None,
        method_args: Optional[List[Any]] = [],
        method_arg_types: Optional[List[str]] = [],
        insight_id: Optional[str] = None,
    ) -> List[Dict]:
        """
        This method is responsible for initiating a synchronous communication with the server, which calls the `comm` method.

        Args:
            epoc (`str`): The epoc ID for the payload struct.
            engine_type (`str`): The engine type that will be called from the Tomcat server. Options are model, storage, database, or vector and are set in NativePyEngineWorker.java.
            engine_id (`str`): The unique identifier of the engine being called. This is passed so the Tomcat server can call Utility.java to find the engine.
            method_name (`str`): The IEngine method name that is available in the `engine_type`.
            method_args (`Optional[List[Any]]`): A list of objects to be sent to the IEngine method as inputs.
            method_arg_types (`Optional[List[str]]`): A list of Java class names that represent the method argument types.
            insight_id (`Optional[str]`): The unique identifier for the insight

        Returns:
            `List[Dict]`: A list that contains the response from the Tomcat server engine.
        """
        epoc = self.get_next_epoc()
        self.comm(
            epoc=epoc,
            engine_type=engine_type,
            engine_id=engine_id,
            method_name=method_name,
            method_args=method_args,
            method_arg_types=method_arg_types,
            insight_id=insight_id,
        )

        new_payload_struct = self.server.monitors.pop(epoc)
        logger.info(new_payload_struct)

        if "ex" in new_payload_struct:
            # if exception, convert it to an Exception and raise it
            raise Exception(new_payload_struct["ex"])
        else:
            logger.info(f"answer is .. {new_payload_struct['payload']}")
            return new_payload_struct["payload"]

    def process_payload(self, payload_struct):
        # try to see if the types are pickle
        # if so unpickle it
        import jsonpickle as jp

        payload_data = None
        if "payload" in payload_struct:
            payload_data = payload_struct["payload"]
        if payload_data is not None and isinstance(payload_data, List):
            for data in payload_data:
                index = payload_data.index(data)
                try:
                    orig_obj = data
                    obj = jp.loads(orig_obj)
                    payload_struct["payload"][index] = obj
                except Exception as e:
                    pass

        return payload_struct



import unittest

from ai_server.server_resources.server_client import ServerClient

from test_base_connection import TestServerClient
from test_run_pixel import TestRunPixel
from test_model import ModelTests
from test_vector import VectorTests
from test_database import DatabaseTests
from test_openai_endpoints import OpenAiEndpointsTests
from test_langchain import LangChainTests
from test_storage import StorageTests
from variables import ACCESS_KEY, SECRET_KEY, ENDPOINT


if __name__ == '__main__':
    test_loader = unittest.TestLoader()
    test_suite = unittest.TestSuite()

    TestServerClient.server_client = ServerClient(
        base=ENDPOINT,
        access_key=ACCESS_KEY,
        secret_key=SECRET_KEY
    )

    run_pixel_test = test_loader.loadTestsFromTestCase(TestRunPixel)
    test_suite.addTest(run_pixel_test)

    model_tests = test_loader.loadTestsFromTestCase(ModelTests)
    test_suite.addTest(model_tests)

    vector_tests = test_loader.loadTestsFromTestCase(VectorTests)
    test_suite.addTest(vector_tests)

    database_tests = test_loader.loadTestsFromTestCase(DatabaseTests)
    test_suite.addTest(database_tests)

    openai_endpoint_tests = test_loader.loadTestsFromTestCase(
        OpenAiEndpointsTests)
    test_suite.addTest(openai_endpoint_tests)

    langchain_tests = test_loader.loadTestsFromTestCase(LangChainTests)
    test_suite.addTest(langchain_tests)

    # No great way to run storage tests locally yet
    if ENDPOINT != "http://localhost:9090/Monolith_Dev/api":
        storage_tests = test_loader.loadTestsFromTestCase(StorageTests)
        test_suite.addTest(storage_tests)

    # run tests
    unittest.TextTestRunner(verbosity=2).run(test_suite)



import unittest
from ai_server.server_resources.server_client import ServerClient
from variables import ACCESS_KEY, SECRET_KEY, ENDPOINT


class TestServerClient(unittest.TestCase):

    server_client = None

    @classmethod
    def setUpClass(cls):
        # Create an instance of ServerClient for testing
        if cls.server_client is None:
            cls.server_client = TestServerClient.login_with_access_keys()

    @staticmethod
    def login_with_access_keys():
        return ServerClient(
            base=ENDPOINT,
            access_key=ACCESS_KEY,
            secret_key=SECRET_KEY
        )


if __name__ == '__main__':
    unittest.main()
